==> libgii-0.1.tar.gz: libgii-0.1/gg/conf.c <==
/* $Id: conf.c,v 1.5 1998/11/21 23:55:52 marcus Exp $
******************************************************************************
  
   LibGG - Configuration handling
  
   Copyright (C) 1997 Jason McMullan	[jmcc@ggi-project.org]
   Copyright (C) 1998 Marcus Sundberg	[marcus@ggi-project.org]
   
   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

******************************************************************************
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include <ggi/gg.h>
#include "plat.h"

/* Private structures */
typedef struct gg_string_list {
	const char *str;
	struct gg_string_list *next;
} gg_string_list;

typedef struct gg_dlconfig {
	const char *name;
	const char *filename;
	gg_string_list *version;
	struct gg_dlconfig *next;
} gg_dlconfig;

static int _ggDoLoad(const char *file, gg_dlconfig **list, int incl)
{
	FILE *inf;
	char root[2048], line[2048], buff[2048], *cp, *bp;
	char *name, *fname;
	gg_dlconfig *dltmp, *dlconf;
	root[0] = 0;

	inf = fopen(file,"r");
	if (inf == NULL)
		return GGI_ENOFILE;

	dlconf = *list;

	while (!feof(inf)) {
		fgets(line, 2048, inf);
		cp = line;

		while (isspace((int)*cp)) cp++;
		if (*cp == 0 || *cp == '#')
			continue;
	
		if (strncmp(cp, ".root:", 6) == 0) {
			cp += 6;
			while (isspace((int)*cp)) cp++;
			name = cp;
			while (*cp && !isspace((int)*cp)) cp++;
			strncpy(root, name, cp-name);
			root[cp-name] = 0;
			continue;
		}
		if (strncmp(cp, ".include", 8) == 0) {
			cp += 8;
			while (isspace((int)*cp)) cp++;
			name = cp;
			while (*cp && !isspace((int)*cp)) cp++;
			strncpy(buff, name, cp-name);
			buff[cp-name] = 0;
			if (incl+1 < 20) {
				*list = dlconf;
				_ggDoLoad(buff, list, incl+1);
				dlconf = *list;
			}
			else {
				fprintf(stderr,"libgg: .include recursion "
						"too deep\n");
			}
		}	
		bp = buff;
		while (!isspace((int)*cp)) *(bp++) = *(cp++);
		*(bp) = 0;
		if ((name = strdup(buff)) == NULL) {
			return GGI_ENOMEM;
		}
		while (isspace((int)*cp)) cp++;
		if (*cp == 0 || *cp == '#') {
			free(name);
			continue;
		}
		bp=buff;
		while (!isspace((int)*cp)) {
			*(bp++) =* (cp++);
		}
		*(bp) = 0;
		if (buff[0] != CHAR_DIRDELIM) {
			if ((fname = (char *) malloc(strlen(buff)
						     +strlen(root)+2))
			    == NULL) {
				return GGI_ENOMEM;
			}
			strcpy(fname,root);
			strcat(fname,STRING_DIRDELIM);
			strcat(fname,buff);
		} else {
			if ((fname = strdup(buff)) == NULL) {
				return GGI_ENOMEM;
			}
		}
		/* Add the config entry....
		 */	
		if ((dltmp = (gg_dlconfig *)malloc(sizeof(gg_dlconfig)))
		    == NULL) {
			return GGI_ENOMEM;
		}
		dltmp->name = name;
		dltmp->filename = fname;
		dltmp->version = NULL;
		dltmp->next = dlconf;
		dlconf = dltmp;

		do {	
			gg_string_list *vtmp;

			while (isspace((int)*cp)) cp++;
			if (*cp == 0 || *cp == '#') 
				break;
			bp = buff;
			while (!isspace((int)*cp)) *(bp++) = *(cp++);
			*(bp) = 0;
			if ((vtmp = (gg_string_list *)malloc(sizeof(gg_string_list)))
			    == NULL 
			    || (vtmp->str = strdup(buff)) == NULL) {
				return GGI_ENOMEM;
			}
			vtmp->next = dltmp->version;
			dltmp->version = vtmp;
		} while (*cp != 0);
			
	}
	fclose(inf);

	*list = dlconf;

	return 0;
}

int ggLoadConfig(const char *file, void **ret)
{
	*ret = NULL;
	return _ggDoLoad(file, (gg_dlconfig **)ret, 0);
}


/*
 * ggFreeConfig()
 *
 * Free the entire config list
 */

void ggFreeConfig(void *conf)
{
	gg_dlconfig *conftmp, *confptr = (gg_dlconfig*) conf;
	
	while (confptr != NULL) {
		gg_string_list *strlist = confptr->version, *strtmp;
		free((char*)confptr->name);
		free((char*)confptr->filename);
		while (strlist != NULL) {
			strtmp = strlist->next;
			free((char*)strlist->str);
			free(strlist);
			strlist = strtmp;
		}
		conftmp = confptr->next;
		free(confptr);
		confptr = conftmp;
	}
}


/* Takes a gg_dlconfig, lib name, and a version. Returns the
 * closest match (Ha!)
 */
const char *ggMatchConfig(const void *conf, const char *name,const char *ver)
{
	gg_dlconfig *dlc,*dtmp,*best = NULL;
	gg_string_list *vtmp;

	dlc = (gg_dlconfig*) conf;

	for (dtmp = dlc; dtmp != NULL; dtmp = dtmp->next) {
		if (strcasecmp(name, dtmp->name)!=0)
			continue;

		if (dtmp->version == NULL) {
			best = dtmp;
			continue;
		}

		for (vtmp = dtmp->version; vtmp != NULL; vtmp = vtmp->next) 
			if (strcmp(ver, vtmp->str) == 0) {
				return dtmp->filename;
			}
	}

	return (best == NULL) ? NULL : best->filename;
}

==> libgii-0.6.tar.gz: libgii-0.6/gg/conf.c <==
/* $Id: conf.c,v 1.6 1999/03/09 17:35:03 marcus Exp $
******************************************************************************
  
   LibGG - Configuration handling
  
   Copyright (C) 1997 Jason McMullan	[jmcc@ggi-project.org]
   Copyright (C) 1998 Marcus Sundberg	[marcus@ggi-project.org]
   
   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

******************************************************************************
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include <ggi/gg.h>
#include "plat.h"

/* Private structures */
typedef struct gg_string_list {
	const char *str;
	struct gg_string_list *next;
} gg_string_list;

typedef struct gg_dlconfig {
	const char *name;
	const char *filename;
	gg_string_list *version;
	struct gg_dlconfig *next;
} gg_dlconfig;

static int
_ggDoLoad(const char *file, gg_dlconfig **list, int incl, char *oldroot)
{
	FILE *inf;
	char root[2048], line[2048], buff[2048], *cp, *bp;
	char *name, *fname;
	gg_dlconfig *dltmp, *dlconf;
	root[0] = '\0';

	inf = fopen(file, "r");
	if (inf == NULL) return GGI_ENOFILE;

	dlconf = *list;

	if (oldroot) {
		strncpy(root, oldroot, 2047);
		root[2047] = '\0';
	}

	while (!feof(inf)) {
		fgets(line, 2048, inf);
		cp = line;

		while (isspace((int)*cp)) cp++;
		if (*cp == '\0' || *cp == '#')
			continue;
	
		if (strncmp(cp, ".root:", 6) == 0) {
			cp += 6;
			while (isspace((int)*cp)) cp++;
			name = cp;
			while (*cp && !isspace((int)*cp)) cp++;
			strncpy(root, name, cp-name);
			root[cp-name] = '\0';
			continue;
		}
		if (strncmp(cp, ".include", 8) == 0) {
			cp += 8;
			while (isspace((int)*cp)) cp++;
			name = cp;
			while (*cp && !isspace((int)*cp)) cp++;
			strncpy(buff, name, cp-name);
			buff[cp-name] = '\0';
			if (incl+1 < 20) {
				*list = dlconf;
				_ggDoLoad(buff, list, incl+1,
					  *root != '\0' ? root : NULL);
				dlconf = *list;
			}
			else {
				fprintf(stderr,"libgg: .include recursion "
						"too deep\n");
			}
		}	
		bp = buff;
		while (!isspace((int)*cp)) *(bp++) = *(cp++);
		*(bp) = '\0';
		if ((name = strdup(buff)) == NULL) {
			return GGI_ENOMEM;
		}
		while (isspace((int)*cp)) cp++;
		if (*cp == '\0' || *cp == '#') {
			free(name);
			continue;
		}
		bp=buff;
		while (!isspace((int)*cp)) {
			*(bp++) =* (cp++);
		}
		*(bp) = '\0';
		if (buff[0] != CHAR_DIRDELIM) {
			if ((fname = (char *) malloc(strlen(buff)
						     +strlen(root)+2))
			    == NULL) {
				return GGI_ENOMEM;
			}
			strcpy(fname,root);
			strcat(fname,STRING_DIRDELIM);
			strcat(fname,buff);
		} else {
			if ((fname = strdup(buff)) == NULL) {
				return GGI_ENOMEM;
			}
		}
		/* Add the config entry....
		 */	
		if ((dltmp = (gg_dlconfig *)malloc(sizeof(gg_dlconfig)))
		    == NULL) {
			return GGI_ENOMEM;
		}
		dltmp->name = name;
		dltmp->filename = fname;
		dltmp->version = NULL;
		dltmp->next = dlconf;
		dlconf = dltmp;

		do {	
			gg_string_list *vtmp;

			while (isspace((int)*cp)) cp++;
			if (*cp == '\0' || *cp == '#') 
				break;
			bp = buff;
			while (!isspace((int)*cp)) *(bp++) = *(cp++);
			*(bp) = '\0';
			if ((vtmp = (gg_string_list *)malloc(sizeof(gg_string_list)))
			    == NULL 
			    || (vtmp->str = strdup(buff)) == NULL) {
				return GGI_ENOMEM;
			}
			vtmp->next = dltmp->version;
			dltmp->version = vtmp;
		} while (*cp != '\0');
			
	}
	fclose(inf);

	*list = dlconf;

	return 0;
}

int ggLoadConfig(const char *file, void **ret)
{
	*ret = NULL;
	return _ggDoLoad(file, (gg_dlconfig **)ret, 0, NULL);
}


/*
 * ggFreeConfig()
 *
 * Free the entire config list
 */

void ggFreeConfig(void *conf)
{
	gg_dlconfig *conftmp, *confptr = (gg_dlconfig*) conf;
	
	while (confptr != NULL) {
		gg_string_list *strlist = confptr->version, *strtmp;
		free((char*)confptr->name);
		free((char*)confptr->filename);
		while (strlist != NULL) {
			strtmp = strlist->next;
			free((char*)strlist->str);
			free(strlist);
			strlist = strtmp;
		}
		conftmp = confptr->next;
		free(confptr);
		confptr = conftmp;
	}
}


/* Takes a gg_dlconfig, lib name, and a version. Returns the
 * closest match (Ha!)
 */
const char *ggMatchConfig(const void *conf, const char *name,const char *ver)
{
	gg_dlconfig *dlc,*dtmp,*best = NULL;
	gg_string_list *vtmp;

	dlc = (gg_dlconfig*) conf;

	for (dtmp = dlc; dtmp != NULL; dtmp = dtmp->next) {
		if (strcasecmp(name, dtmp->name)!=0)
			continue;

		if (dtmp->version == NULL) {
			best = dtmp;
			continue;
		}

		for (vtmp = dtmp->version; vtmp != NULL; vtmp = vtmp->next) 
			if (strcmp(ver, vtmp->str) == 0) {
				return dtmp->filename;
			}
	}

	return (best == NULL) ? NULL : best->filename;
}
